#ifndef COMPLEX_H
#define COMPLEX_H

#include <cmath>
#include <iostream>
#include <cassert>

#include "vector.h"

class complex_number : public vector2
{
public:
    /*
     * takes in the real and imaginary
     * components values e.g. z = a + bi
     */
    complex_number(float a, float b)
    {
        set(a,b);
    }

    complex_number()
    {
        set(0,0);
    }

    void mult(const complex_number& y, const complex_number& z)
    {
        // ensure that the caller is not passed in as a parameter
        assert(&y != this && &z != this);

        p[0] = y[0]*z[0] - y[1]*z[1];
        p[1] = y[0]*z[1] + y[1]*z[0];
    }
    
    void postmult(const complex_number& z)
    {
        complex_number tmp;
        tmp.set(*this);
        mult(tmp, z);
    }

    void set(const complex_number& z)
    {
        vector2::set(z);
    }

    void get(float &a, float &b) const
    {
        a = p[0];
        b = p[1];
    }

    void set(float a, float b) 
    {
        p[0] = a;
        p[1] = b;
    }
    
    float norm() const
    {
        return p[0]*p[0]+p[1]*p[1];
    }
}; 

#endif
