#include "gl_viewer.h"

#include <cassert>
#include <iostream>
#include <cstdio>

using namespace std;

gl_viewer* gl_viewer::singleton = NULL; 

gl_viewer::gl_viewer()
{
    assert(!singleton); // ensure only one instance is created
    singleton = this;
}

gl_viewer::~gl_viewer()
{
    singleton = NULL;
}

void gl_viewer::init(int argc, char *argv[], int width, int height)
{
    // set up glut
    glutInit(&argc, argv);

    // create a 24-bit double-buffered window
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH); 

    glutInitWindowSize(width, height);
    glutCreateWindow("OGL 3D Viewer");

    // called when Glut needs to display
    glutDisplayFunc(&gl_viewer::glut_display_event_wrapper);

    // called when Glut has detected a mouse click
    glutMouseFunc(&gl_viewer::glut_mouse_click_event_wrapper);
    
    // called when Glut has detected mouse motion
    glutMotionFunc(&gl_viewer::glut_mouse_move_event_wrapper);

    // called when Glut has key input
    glutKeyboardFunc(&gl_viewer::glut_keyboard_event_wrapper);
    
    // called when GLUT has nothing to do
    //glutIdleFunc(&gl_viewer::glut_display_event_wrapper);

    // called when the window is resized
    glutReshapeFunc(&gl_viewer::glut_reshape_event_wrapper);

    // clear our background to black when glClear is called
    glClearColor(0.9, 0.9, 1.0, 0); 
    
    this->width = width;
    this->height = height;
    first_click = true;

    singleton->init_event();
}

void gl_viewer::run()
{
    // pass execution to Glut. Now Glut is in control of the main loop.
    glutMainLoop();
}

void gl_viewer::glut_display_event_wrapper()
{
    // clear our color buffer
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
    
    // call our gl_viewers draw event and render to back buffer
    singleton->draw_event();

    // swap the back buffer with the front (user always sees front
    // buffer on display)
    glutSwapBuffers();
}

void gl_viewer::glut_mouse_click_event_wrapper(
    int button, int state, int x, int y
    )
{
    singleton->mouse_click_event(
        (mouse_button)button, (mouse_button_state)state, 
        x, singleton->height - y
    );
}

void gl_viewer::glut_mouse_move_event_wrapper(
    int x, int y
    )
{
    singleton->mouse_move_event(x, singleton->height - y);
}

void gl_viewer::glut_keyboard_event_wrapper(unsigned char key, int x, int y)
{
    singleton->keyboard_event(key, x, singleton->height - y);
}

void gl_viewer::glut_reshape_event_wrapper(int width, int height)
{
    singleton->width = width;
    singleton->height = height;

    // adjust the view frustrum and viewport 
    // to reflect the new window dimensions
    glViewport(0, 0, width, height);
    
    if (height == 0) height = 1;

    // set up perspective projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, width, 0, height);
    glMatrixMode(GL_MODELVIEW);
}

